{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# V1\n",
    "\n",
    "## Punto 1: Aplicación de los 4 Pilares de la Programación Orientada a Objetos (POO) [30 pts]\n",
    "\n",
    "Desarrolla una aplicación en Java donde se implemente una clase llamada **\"Producto\"** para simular la gestión de productos en una tienda. Debes utilizar los cuatro pilares fundamentales de la programación orientada a objetos: **abstracción**, **encapsulamiento**, **herencia**, y **polimorfismo**.\n",
    "\n",
    "- Crea una interfaz llamada `Item` que tenga los atributos `nombre` y `precio`, además de dos métodos abstractos `mostrarDetalles()` y `estaDisponible()`.\n",
    "- La clase `Producto` debe heredar de `Item` e incluir atributos como `categoria` y `cantidadEnStock`.\n",
    "- Utiliza encapsulamiento para proteger los atributos, permitiendo acceso a través de los métodos `getters` y `setters`.\n",
    "- Implementa una subclase llamada `ProductoElectronico` que sobrescriba el método `mostrarDetalles()` e incluya un atributo adicional `garantia`.\n",
    "- Aplica polimorfismo mostrando cómo un objeto de tipo `ProductoElectronico` puede ser tratado como un `Item`, verifica que las funciones `mostrarDetalles()` funciona y que da diferente resultados dependiendo el objeto*que la llame.\n",
    "\n",
    "**Requisitos adicionales:**\n",
    "- Implementa un constructor en cada clase.\n",
    "- Instancia objetos en el método `main()` y muestra ejemplos de polimorfismo en acción.\n",
    "\n",
    "**Ejemplo de salida esperada:**\n",
    "```text\n",
    "Producto: Laptop, Precio: $1200, Categoría: Electrónica, Garantía: 2 años.\n",
    "Producto: Libro, Precio: $20, Categoría: Literatura.\n",
    "```"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "interface Item{\n",
    "  // como es una interfaz estos son public y final, no se\n",
    "  // pueden cambiar y tienen que ser definidos sus valores\n",
    "  public String nombre = \"Item\";\n",
    "  public double precio = 0;\n",
    "\n",
    "  //public Item(){}\n",
    "\n",
    "  public abstract boolean estaDisponible();\n",
    "  public abstract void mostartDetallers();\n",
    "}\n",
    "\n",
    "class Producto implements Item{\n",
    "  private String categoria;\n",
    "  private int cantidadEnStock;\n",
    "  // se puede sobreescribir nombre y precio pero toca desde aca\n",
    "  //public String nombre;\n",
    "  //public double precio;\n",
    "\n",
    "  public Producto(String categoria, int cantidadEnStock){\n",
    "    this.categoria = categoria;\n",
    "    this.cantidadEnStock = cantidadEnStock;\n",
    "  }\n",
    "\n",
    "  public void setCategoria(String categoria){\n",
    "    this.categoria = categoria;\n",
    "  }\n",
    "  public String getCategoria(){\n",
    "    return categoria;\n",
    "  }\n",
    "\n",
    "  public void setCantidadEnStock(int cantidadEnStock){\n",
    "    this.cantidadEnStock = cantidadEnStock;\n",
    "  }\n",
    "  public int getCantidadEnStock(){\n",
    "    return cantidadEnStock;\n",
    "  }\n",
    "\n",
    "  @Override\n",
    "  public boolean estaDisponible(){\n",
    "    if(cantidadEnStock>0){return true;}\n",
    "    else{return false;}\n",
    "  };\n",
    "  @Override\n",
    "  public void mostartDetallers(){\n",
    "    System.out.println(\"El producto es %s y tiene un precio de %.1f.%n\".formatted(nombre, precio));\n",
    "    System.out.println(\"Pertenece a la categoria %s y hay %d en la tienda.%n\".formatted(categoria, cantidadEnStock));\n",
    "  };\n",
    "}\n",
    "\n",
    "class ProductoElectronico extends Producto{\n",
    "  public int garantia;\n",
    "\n",
    "  public ProductoElectronico(String categoria, int cantidadEnStock, int garantia){\n",
    "    super(categoria, cantidadEnStock);\n",
    "    this.garantia = garantia;\n",
    "  }\n",
    "\n",
    "  @Override\n",
    "  public boolean estaDisponible(){\n",
    "    if(getCantidadEnStock()>0){return true;}\n",
    "    else{return false;}\n",
    "  };\n",
    "  @Override\n",
    "  public void mostartDetallers(){\n",
    "    System.out.println(\"El producto es %s y tiene un precio de %.1f.%n\".formatted(nombre, precio));\n",
    "    System.out.println(\"Tiene una garantia de %d meses.%n\".formatted(garantia));\n",
    "  };\n",
    "}\n",
    "\n",
    "/*o se podia\n",
    "class ProductoElectronico implements Item{\n",
    "  // codigo similar a producto\n",
    "}\n",
    "*/"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 54,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "true\n",
      "El producto es Item y tiene un precio de 0,0.\n",
      "Tiene una garantia de 12 meses.\n",
      "\n",
      "false\n",
      "El producto es Item y tiene un precio de 0,0.\n",
      "Pertenece a la categoria Ropa y hay 0 en la tienda.\n"
     ]
    }
   ],
   "source": [
    "class TestProductos{\n",
    "  public static void main(String[] args){\n",
    "    ProductoElectronico nevera = new ProductoElectronico(\"Electrodomesticos\", 10, 12);\n",
    "    System.out.println(nevera.estaDisponible());\n",
    "    nevera.mostartDetallers();\n",
    "\n",
    "    System.out.println(\"\");\n",
    "    Producto camisa = new Producto(\"Ropa\", 0);\n",
    "    System.out.println(camisa.estaDisponible());\n",
    "    camisa.mostartDetallers();\n",
    "  }\n",
    "}\n",
    "\n",
    "TestProductos testProductos = new TestProductos();\n",
    "testProductos.main(null);"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Punto 2: Manipulación de Cadenas [30 pts]\n",
    "\n",
    "Desarrolla un programa en Java que permita a un usuario ingresar una cadena de texto y, con ella, realice las siguientes operaciones:\n",
    "\n",
    "1. **Eliminar espacios en blanco**: Implementa un método que elimine todos los espacios en blanco de la cadena ingresada.\n",
    "2. **Contar las vocales**: Implementa un método que cuente cuántas vocales (a, e, i, o, u) tiene la cadena.\n",
    "3. **Reemplazar caracteres**: Implementa un método que permita al usuario reemplazar una letra específica por otra.\n",
    "\n",
    "El programa debe interactuar con el usuario y mostrar los resultados para cada una de las operaciones. Usa los métodos de la clase `String` para manipular las cadenas.\n",
    "\n",
    "**Ejemplo de entrada:**\n",
    "\n",
    "```text\n",
    "Ingresa una cadena: amor\n",
    "```\n",
    "\n",
    "**Ejemplo de salida esperada:**\n",
    "\n",
    "  ```text\n",
    "  Cadena sin espacios: amor\n",
    "  Número de vocales: 2\n",
    "\n",
    "  Ingresa una letra para reemplazar: a\n",
    "  Ingresa la nueva letra: e\n",
    "  Cadena modificada: emor\n",
    "  ```"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [],
   "source": [
    "public String RemoverEspacios(String cadena){\n",
    "  String cadena_sin_espacios = \"\";\n",
    "  for(int i=0; i<cadena.length(); i++){\n",
    "    if(cadena.charAt(i) != ' '){\n",
    "      cadena_sin_espacios += cadena.charAt(i);\n",
    "    }\n",
    "  }\n",
    "  return cadena_sin_espacios;\n",
    "}\n",
    "\n",
    "public int contarVocales(String cadena){\n",
    "  int vocales = 0;\n",
    "  for(int i=0; i<cadena.length(); i++){\n",
    "    \n",
    "    if(cadena.charAt(i) == 'a' || cadena.charAt(i) == 'e' || cadena.charAt(i) == 'i' ||\n",
    "       cadena.charAt(i) == 'o'|| cadena.charAt(i) == 'u'){\n",
    "      vocales += 1;\n",
    "    }\n",
    "  }\n",
    "  return vocales;\n",
    "}\n",
    "\n",
    "public String ReemplazarLetra(String cadena){\n",
    "  System.out.print(\"Ingresa una letra para reemplazar: \");\n",
    "  // Scanner myObj = new Scanner(System.in);\n",
    "  // String letra1 = myObj.nextLine();  // Read user input\n",
    "  String letra1 = \"a\";\n",
    "  System.out.println(letra1);\n",
    "\n",
    "  System.out.print(\"Ingresa la nueva letra: \");\n",
    "  //String letra2 = myObj.nextLine();  // Read user input\n",
    "  String letra2 = \"e\";\n",
    "  System.out.println(letra2);\n",
    "  \n",
    "  return cadena.replace(letra1, letra2);\n",
    "\n",
    "}"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Ingresa una cadena:  el amor es para todos y todas\n",
      "\n",
      "Cadena sin espacios: elamoresparatodosytodas\n",
      "Número de vocales: 10\n",
      "\n",
      "Ingresa una letra para reemplazar: a\n",
      "Ingresa la nueva letra: e\n",
      "Cadena modificada:  el emor es pere todos y todes\n"
     ]
    }
   ],
   "source": [
    "import java.util.Scanner;\n",
    "\n",
    "//Scanner myObj = new Scanner(System.in);\n",
    "System.out.print(\"Ingresa una cadena: \");\n",
    "String cadena = \" el amor es para todos y todas\"; \n",
    "//String cadena = myObj.nextLine();  // Read user input\n",
    "System.out.println(cadena+\"\\n\");\n",
    "\n",
    "System.out.println(\"Cadena sin espacios: \"+RemoverEspacios(cadena));\n",
    "System.out.println(\"Número de vocales: \"+contarVocales(cadena)+\"\\n\");\n",
    "\n",
    "String cadena_modificada = ReemplazarLetra(cadena);\n",
    "System.out.println(\"Cadena modificada: \"+cadena_modificada);"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Punto 3: Arreglos Bidimensionales y Búsquedas [40 pts]\n",
    "\n",
    "En este ejercicio, debes crear un programa para gestionar las ventas de productos en una tienda. Implementa un sistema que use un **arreglo bidimensional** para almacenar las ventas de productos por día de la semana, donde cada fila corresponde a un producto y cada columna a un día de la semana.\n",
    "\n",
    "1. Declara un arreglo bidimensional donde las filas representen productos y las columnas las ventas diarias:\n",
    "   \n",
    "   ```java\n",
    "   int[][] ventas = {\n",
    "       {12, 15, 20, 10, 25, 30, 40}, // ProductoA\n",
    "       {5, 7, 8, 6, 10, 11, 15},     // ProductoA\n",
    "       {3, 4, 2, 5, 6, 8, 9}         // ProductoA\n",
    "   };\n",
    "   // este codigo es equivalente a\n",
    "   int[] ProductoA = {12, 15, 20, 10, 25, 30, 40};\n",
    "   int[] ProductoB = {5, 7, 8, 6, 10, 11, 15};\n",
    "   int[] ProductoC = {3, 4, 2, 5, 6, 8, 9};\n",
    "   ```\n",
    "   \n",
    "2. **Implementa un método** que calcule el total de ventas por producto durante la semana.\n",
    "   \n",
    "3. **Implementa un método** para encontrar el día en que se registraron las mayores ventas para cada producto.\n",
    "   \n",
    "4. **Implementa un método de búsqueda** que permita encontrar cuántas ventas de un producto específico hubo en un día dado, devolviendo la posición (producto, día). Si no se encuentra, el programa debe informarlo.\n",
    "\n",
    "5. **Crea un menú** para que el usuario pueda:\n",
    "   - Calcular el total de ventas por producto durante la semana.\n",
    "   - Mostrar el día con mayores ventas por producto.\n",
    "   - Buscar el número de ventas de un producto en un día específico.\n",
    "\n",
    "**Ejemplo de salida esperada:**\n",
    "```text\n",
    "Menú:\n",
    "1. Calcular total de ventas por producto\n",
    "2. Mostrar día con mayores ventas por producto\n",
    "Opción: 1\n",
    "\n",
    "Total de ventas para el Producto 1: 152\n",
    "Total de ventas para el Producto 2: 62\n",
    "Total de ventas para el Producto 3: 37\n",
    "```"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 15,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Menú:\n",
      "  1. Calcular total de ventas por producto\n",
      "  2. Mostrar día con mayores ventas por producto\n",
      "Opción: 1\n",
      "Del ProductoA se vendieron en toda la semana 152\n",
      "Del ProductoB se vendieron en toda la semana 62\n",
      "Del ProductoC se vendieron en toda la semana 37\n",
      "Del ProductoD se vendieron en toda la semana 47\n",
      "\n"
     ]
    }
   ],
   "source": [
    "int[] ProductoA = {12, 15, 20, 10, 25, 30, 40};\n",
    "int[] ProductoB = {5, 7, 8, 6, 10, 11, 15};\n",
    "int[] ProductoC = {3, 4, 2, 5, 6, 8, 9};\n",
    "int[] ProductoD = {3, 4, 2, 15, 6, 8, 9};\n",
    "\n",
    "public int VentasPorSemana(int[] Producto){\n",
    "  int sum = 0;\n",
    "  for(int i=0; i<Producto.length; i++ ){\n",
    "    sum += Producto[i];\n",
    "  }\n",
    "  return sum;\n",
    "}\n",
    "\n",
    "public String MayorVenta(int[] Producto){\n",
    "  int mayor = 0;\n",
    "  int mayor_index = 0;\n",
    "  String[] dias_semana = {\"Lunes\", \"Martes\", \"Miercoles\", \"Jueves\", \"Viernes\", \"Sabado\", \"Domingo\"};\n",
    "\n",
    "  for(int i=0; i<Producto.length; i++ ){\n",
    "    if(Producto[i]>mayor){\n",
    "      mayor = Producto[i];\n",
    "      mayor_index = i;\n",
    "    }\n",
    "  }\n",
    "  return dias_semana[mayor_index];\n",
    "}\n",
    "\n",
    "import java.util.Scanner;\n",
    "public void Menu(){\n",
    "  System.out.print(\"\"\"\n",
    "Menú:\n",
    "  1. Calcular total de ventas por producto\n",
    "  2. Mostrar día con mayores ventas por producto\n",
    "\"\"\");\n",
    "  // System.ount.println(\"Opción: \");\n",
    "  //Scanner myObj = new Scanner(System.in);\n",
    "  //int opcion = myObj.nextInt();  // Read user input\n",
    "  int opcion = 1;\n",
    "  System.out.println(\"Opción: \"+opcion);\n",
    "\n",
    "  if(opcion==1){\n",
    "    System.out.println(\"\"\"\n",
    "        Del ProductoA se vendieron en toda la semana %d\n",
    "        Del ProductoB se vendieron en toda la semana %d\n",
    "        Del ProductoC se vendieron en toda la semana %d\n",
    "        Del ProductoD se vendieron en toda la semana %d\n",
    "        \"\"\".formatted(VentasPorSemana(ProductoA), VentasPorSemana(ProductoB),\n",
    "                      VentasPorSemana(ProductoC), VentasPorSemana(ProductoD)));\n",
    "  }\n",
    "  else if(opcion==2){\n",
    "    System.out.println(\"\"\"\n",
    "          Para el ProductoA el día de la semana con más ventas es %s\n",
    "          Para el ProductoA el día de la semana con más ventas es %s\n",
    "          Para el ProductoA el día de la semana con más ventas es %s\n",
    "          Para el ProductoA el día de la semana con más ventas es %s\n",
    "        \"\"\".formatted(MayorVenta(ProductoA), MayorVenta(ProductoB),\n",
    "                      MayorVenta(ProductoC), MayorVenta(ProductoD)));\n",
    "  }\n",
    "}\n",
    "\n",
    "Menu();\n",
    "//MayorVenta(ProductoA);\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 13,
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Producto A vendio 25 el dia Viernes.\n",
      "\n",
      "Producto C vendio 4 el dia Martes.\n",
      "\n"
     ]
    }
   ],
   "source": [
    "// Este punto 4 no tocaba hacerlo\n",
    "public int DiaSemanaIndex(String dia){\n",
    "  String[] dias_semana = {\"Lunes\", \"Martes\", \"Miercoles\", \"Jueves\", \"Viernes\", \"Sabado\", \"Domingo\"};\n",
    "  int index = 0;\n",
    "  for(int i=0; i<dias_semana.length; i++ ){\n",
    "    if(dias_semana[i]==dia){\n",
    "      index = i;\n",
    "    }\n",
    "  }\n",
    "  return index;\n",
    "}\n",
    "\n",
    "public void VentasProductoDia(String Producto, String dia){\n",
    "  int index_dia = DiaSemanaIndex(dia);\n",
    "  if(Producto == \"A\"){\n",
    "    System.out.println(\"Producto %s vendio %d el dia %s.%n\".formatted(Producto, ProductoA[index_dia], dia));\n",
    "  }\n",
    "  else if(Producto == \"B\"){\n",
    "    System.out.println(\"Producto %s vendio %d el dia %s.%n\".formatted(Producto, ProductoB[index_dia], dia));\n",
    "  }\n",
    "  else if(Producto == \"C\"){\n",
    "    System.out.println(\"Producto %s vendio %d el dia %s.%n\".formatted(Producto, ProductoC[index_dia], dia));\n",
    "  }\n",
    "  else{\n",
    "    // no existe el producto\n",
    "    System.out.println(\"Producto %s no encontrado.%n\".formatted(Producto));\n",
    "  }\n",
    "}\n",
    "\n",
    "VentasProductoDia(\"A\", \"Viernes\");\n",
    "VentasProductoDia(\"C\", \"Martes\");\n",
    "//DiaSemanaIndex(\"Domingo\")"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Java",
   "language": "java",
   "name": "java"
  },
  "language_info": {
   "codemirror_mode": "java",
   "file_extension": ".jshell",
   "mimetype": "text/x-java-source",
   "name": "java",
   "pygments_lexer": "java",
   "version": "21.0.4+7-Ubuntu-1ubuntu224.04"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
